//@author: vux
//@help: builds round rects in vertex shader
//@tags: color
//@credits: 

StructuredBuffer<float4> sbWorld : register(t0);
StructuredBuffer<uint> sbStartIndex;
StructuredBuffer<float2> sbInnerOuter : register(t2);

cbuffer cbPerDraw : register(b0)
{
	float4x4 tVP : VIEWPROJECTION;
};

cbuffer cbFixed : register(b2)
{
	float2 initialInner; //This is original innerRadius
	float innerShift;
}

cbuffer cbPerObj : register( b1 )
{
	int drawIndex : DRAWINDEX;
	float4x4 tW : WORLD;
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
};

void VS(float4 objPos : POSITION, uint iv : SV_VertexID,uint ii : SV_InstanceID, out float4 screenPos : SV_Position)
{
	uint idx = sbStartIndex[drawIndex] + ii;
	float4 wps = sbWorld[idx];
	
	bool outer = iv > 4;
	float2 innerOuter = sbInnerOuter[idx];
	innerOuter.x -= innerShift;
	
	//case id = 1 (outer side, scale down back to center
	float2 recenter = outer ? initialInner : 0.0f;
	float2 scale = outer ? innerOuter.yy : innerOuter.xx * (1.0f / initialInner);
	float2 postScale = outer ? innerOuter.xx : 0.0f;

	float2 transformedPos = objPos.xy - sign(objPos.xy) * recenter;
	transformedPos *= scale;
	transformedPos += (innerOuter.xx * sign(objPos.xy)) * outer;

	float4 wp = float4(transformedPos,0,1);
	wp.xyz *= wps.www;
	wp.xyz += wps.xyz;
	screenPos = mul(wp,mul(tW,tVP));

}

float4 PS(float4 p : SV_Position): SV_Target
{
    float4 col = cAmb;
    return col;
}

technique11 Render
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetPixelShader( CompileShader( ps_5_0, PS() ) );
	}
}





